<?php
/**
 * Plumrocket Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the End-user License Agreement
 * that is available through the world-wide-web at this URL:
 * http://wiki.plumrocket.net/wiki/EULA
 * If you are unable to obtain it through the world-wide-web, please
 * send an email to support@plumrocket.com so we can send you a copy immediately.
 *
 * @package     Plumrocket Private Sales
 * @copyright   Copyright (c) 2016 Plumrocket Inc. (http://www.plumrocket.com)
 * @license     http://wiki.plumrocket.net/wiki/EULA  End-user License Agreement
 */

class Step2 extends AbstractStep
{

	private $array = []; // =))


	public function run()
	{
		if (Mage::app()->getRequest()->getParam('redirected')) {
			$this->temp();
			$this->configure();
			$this->success = "Upgrading process is successfully completed!";
		} else {


			if ($this->checkNewPrivatesale() ) {
				try {
					$this->createEvents();

					$resource = Mage::getSingleton("core/resource");
					$setup = new Mage_Eav_Model_Entity_Setup('core_setup');
					$setup->run("DELETE FROM " . $resource->getTableName('core_resource') . " WHERE `code` = 'privatesales_setup'");

					$basePath = Mage::getBaseDir();
					exec('rm -rf ' . $basePath . '/var/cache');

					Mage::reset();

					$mageRunCode = isset($_SERVER['MAGE_RUN_CODE']) ? $_SERVER['MAGE_RUN_CODE'] : '';

					/* Run store or run website */
					$mageRunType = isset($_SERVER['MAGE_RUN_TYPE']) ? $_SERVER['MAGE_RUN_TYPE'] : 'store';

					ob_start();

					try {
						Mage::run('default');
					} catch (Exception $e) {
						Mage::log($e->getMessage(), null, 'migration.log');
					}

					ob_clean();
					$this->temp($this->array);
					unlink('../maintenance.flag');

					$this->configure();

					$this->success = "<h1>Upgrading process is successfully completed!</h1>";
				} catch (Exception $e)
				{
					echo $e->getMessage();
					exit;
				}
			}
		}
	}


	private function configure()
	{
		$stores = Mage::app()->getStores();

		Mage::getConfig()->saveConfig('privatesales/general/enable', '1', 'default', 0);
		Mage::getConfig()->reinit();
		foreach ($stores as $store) {

			$storeId = $store->getId();
			$categories = Mage::getModel('catalog/category')
			    ->getCollection();

			foreach ($categories as $cat) {
				$_cat = Mage::getModel('catalog/category')->load($cat->getId());
				$_cat->save();
			}

			$homepage = Mage::getStoreConfig('web/default/cms_home_page', $storeId);
			$package = Mage::getStoreConfig('design/package/name', $storeId);

			if (strpos($package, 'private_sales') !== false) {
				Mage::getConfig()->saveConfig('web/default/cms_home_page', 'privatesales-homepage', 'stores', $storeId);
				Mage::getConfig()->saveConfig('design/package/name', 'rwd', 'stores', $storeId);

				Mage::getModel('cms/page')->load('privatesales-homepage', 'identifier')
					->setStatus(1)
					->save();
			}

			$page = Mage::getModel('cms/page')->load($homepage, 'identifier');

			if ($page->getData('root_template') == 'empty') {


				$page->setData('root_template', 'one_column')
					->save();
			}
		}
		Mage::getConfig()->reinit();
	}


	public function temp($array = null)
	{
		if (is_null($array)) {
			$array = json_decode(file_get_contents('temp.json'), 1);
		}

		foreach ($array as $catId => $item) {

			$category = Mage::getModel('catalog/category')->load($catId);
			$type = $item['type'];

			$save = false;
			if ($type != 3 && ($item['date_start'] !== null || $item['date_end'] !== null)) {
				$save = true;
				$category
					// ->setStoreId(0)
				    ->setPrivatesaleDateStart($array[$category->getId()]['date_start'])
					->setPrivatesaleDateEnd($array[$category->getId()]['date_end']);
			}

			if ($item['image']) {
				$save = true;
				$category->setData('privatesale_event_image', $item['image']);
			}

			if ($type == 2) {

				$save = true;
				$category->setData('display_mode', Plumrocket_Privatesales_Model_Attribute_Source_Mode::FLASH_SALE_HOMEPAGE)
					->setStoreId(0);
			} elseif ($type == 3) {
				$childs = $category->getChildrenCategories();

				foreach ($childs as $child) {
					$child->setPrivatesaleDateStart( $array[$category->getId()]['date_start'])
					    ->setStoreId(0)
						->setPrivatesaleDateEnd( $array[$category->getId()]['date_end'])
						->save();
				}
			}

			if ($save) {
				$category->save();
			}
		}
	}


	private function createEvents()
	{

		$stores = Mage::app()->getStores();

		$categories = Mage::getModel('catalog/category')->getCollection()
			->addAttributeToSelect('privatesale_type')
			->addAttributeToSelect('image')
			->addAttributeToSelect('privatesale_date_start')
			->addAttributeToSelect('privatesale_date_end');

		foreach ($categories as $category) {

			$array[$category->getId()] = array(
				'type' => $category->getData('privatesale_type'),
				'date_start' => $category->getData('privatesale_date_start'),
				'date_end' => $category->getData('privatesale_date_end'),
				'image'	=> $category->getData('image')
			);
		}

		$json = json_encode($array);
		file_put_contents('temp.json', $json);
 		$this->removeOldAttributes();

 		$this->array = $array;

	}

	private function removeOldAttributes()
	{
		$setup = new Mage_Eav_Model_Entity_Setup('core_setup');
		foreach($this->config->attributes as $entityTypeName => $attributeNames){
			$entityTypeId = $setup->getEntityTypeId($entityTypeName);
			foreach($attributeNames as $attributeName){
				$setup->removeAttribute($entityTypeId, $attributeName);
			}
		}
	}

	private function checkNewPrivatesale()
	{
		$version = Mage::getConfig()->getNode()->modules->Plumrocket_Privatesales->version;

		if ($version === null) {
			$this->error = 'Private Sales and Flash Sales Extension is not installed';
			return false;
		}

		if (version_compare($version, '3.0.0', '<')) {
			$this->error = 'Old version of the extension still exists. Please run step 1';
			return false;
		}

		return true;
	}


}